.\" luac.man,v 1.19 2000/09/04 21:41:28 lhf Exp
.TH LUAC 1 "2000/09/04 21:41:28"
.SH NAME
luac \- Lua compiler
.SH SYNOPSIS
.B luac
[
.I options
] [
.I filenames
]
.SH DESCRIPTION
.B luac
is the Lua compiler.
It translates programs written in the Lua programming language
into binary files that can be loaded and executed with
.B lua_dofile
in C or with
.B dofile
in Lua.
.LP
The main advantages of precompiling chunks are:
faster loading,
protecting source code from user changes,
and
off-line syntax error detection.
.LP
Pre-compiling does not imply faster execution
because in Lua chunks are always compiled into bytecodes before being executed.
.B luac
simply allows those bytecodes to be saved in a file for later execution.
.LP
.B luac
produces a single output file containing the bytecodes
for all source files given.
By default,
the output file is named
.BR luac.out ,
but you can change this with the
.B \-o
option.
.LP
The binary files created by
.B luac
are portable to all architectures with the same word size.
This means that
binary files created on a 32-bit platform (such as Intel)
can be read without change in another 32-bit platform (such as Sparc),
even if the byte order (``endianess'') is different.
On the other hand,
binary files created on a 16-bit platform cannot be read in a 32-bit platform.
.LP
In the command line,
you can mix
text files containing Lua source and
binary files containing precompiled chunks.
This is useful to combine several precompiled chunks,
even from different (but compatible) platforms,
into a single precompiled chunk.
.LP
You can use
.B "\-"
to indicate
.I stdin
as a source file.
.LP
The internal format of the binary files produced by
.B luac
may change when a new version of Lua is released.
We try to maintain compatibility even for binary files,
but it is not always possible.
So,
save the source files of all Lua programs that you precompile.
.LP
.SH OPTIONS
Options must be separate.
.TP
.B \-l
produce a listing of the compiled bytecode for Lua's virtual machine.
Listing bytecodes is useful to learn about Lua's virtual machine.
If no files are given, then
.B luac
loads
.B luac.out
and lists its contents.
.TP
.BI \-o " file"
output to
.IR file ,
instead of the default
.BR luac.out .
The output file can be a source file because
all files are loaded before the output file is written.
.TP
.B \-p
load files but do not generate any output file.
Used mainly for syntax checking or testing precompiled chunks:
corrupted files will probably generate errors when loaded.
For a thourough integrity test,
use
.BR \-t .
.TP
.B \-t
perform a thourough integrity test of precompiled chunks.
Code that passes this test is completely safe,
in the sense that it will not break the interpreter.
However,
there is no guarantee that such code does anything sensible.
(None can be given, because the halting problem is unsolvable.)
If no files are given, then
.B luac
loads
.B luac.out
and tests its contents.
.TP
.B \-v
print version information.
.SH FILES
.TP 15
.B luac.out
default output file
.SH "SEE ALSO"
.BR lua (1)
.br
http://www.tecgraf.puc-rio.br/lua/
.SH DIAGNOSTICS
Error messages should be self explanatory.
.SH AUTHORS
L. H. de Figueiredo,
R. Ierusalimschy and
W. Celes
(lua@tecgraf.puc-rio.br)
.\" EOF
